/*****************************************************************************/
/*!
 * \file theorem_manager.cpp
 * 
 * Author: Sergey Berezin
 * 
 * Created: Feb 11 02:39:35 GMT 2003
 *
 * <hr>
 * Copyright (C) 2003 by the Board of Trustees of Leland Stanford
 * Junior University and by New York University. 
 *
 * License to use, copy, modify, sell and/or distribute this software
 * and its documentation for any purpose is hereby granted without
 * royalty, subject to the terms and conditions defined in the \ref
 * LICENSE file provided with this distribution.  In particular:
 *
 * - The above copyright notice and this permission notice must appear
 * in all copies of the software and related documentation.
 *
 * - THE SOFTWARE IS PROVIDED "AS-IS", WITHOUT ANY WARRANTIES,
 * EXPRESSED OR IMPLIED.  USE IT AT YOUR OWN RISK.
 * 
 * <hr>
 * 
 */
/*****************************************************************************/
// File: theorem_manager.cpp
//
// AUTHOR: Sergey Berezin, 07/05/02
//
// Defines some functions for class TheoremManager.  They are not
// inlined becaule they use ExprManager (expr_manager.h), which
// includes theorem_manager.h.
// 
///////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include "theorem_manager.h"
#include "expr_manager.h"
#include "vcl.h"
#include "debug.h"
#include "theorem_value.h"
#include "memory_manager_chunks.h"
#include "memory_manager_malloc.h"

using namespace std;
using namespace CVCL;

// ExprManager is not initialized in vcl yet when we are created; we
// use d_em as our local cache to fetch the EM when our getEM() is
// first called.

TheoremManager::TheoremManager(VCL* vcl, bool withProof,
			       bool withAssump)
  : d_vcl(vcl), d_em(NULL),
    d_withProof(withProof), d_withAssump(withAssump), d_flag(1),
    d_active(true) {
  DebugAssert(!withProof || withAssump, 
	      "TheoremManager(): proofs without assumptions are not allowed");
  if(vcl->getFlags()["mm"].getString() == "chunks") {
    d_mm = new MemoryManagerChunks(sizeof(TheoremValue));
    d_rwmm = new MemoryManagerChunks(sizeof(RWTheoremValue));
    d_reflmm = new MemoryManagerChunks(sizeof(ReflexivityTheoremValue));
  } else {
    d_mm = new MemoryManagerMalloc();
    d_rwmm = new MemoryManagerMalloc();
    d_reflmm = new MemoryManagerMalloc();
  }
}

TheoremManager::~TheoremManager() {
    delete d_mm;
    delete d_rwmm;
    delete d_reflmm;
}


void TheoremManager::clear() {
  d_active=false;
}


void
TheoremManager::init() {
  d_em = d_vcl->getEM();
  d_em->newKind(PF_APPLY, "|-");
  d_em->newKind(PF_HOLE, "**");
}

//used only by theorytest
void TheoremManager::init(ExprManager* em) {
  d_em = em;
  d_em->newKind(PF_APPLY, "|-");
  d_em->newKind(PF_HOLE, "**");
}


ExprManager*
TheoremManager::getEM() {
//   if(d_em==NULL)
//     d_em = d_vcl->getEM();
  return d_em;
}
